from ..FormComponentType import FormComponentType


class MicrosoftFormComponentType(FormComponentType):
    """FOrmComponentType specific for Microsoft form"""
    SELECT = 'SELECT'
    DATE_PICKER = 'DATE_PICKER'
    RADIO = 'RADIO'
    TEXT = 'TEXT'
